declare type NewContent = Record<string, any>;
declare type NewContentWithId = NewContent & {
    id: number | string;
};
declare type ArrayElement = NewContentWithId & {
    index: number;
};
declare class Array {
    private _name;
    constructor(name: string);
    /**
     * Gets a copy of the original array content.
     * It uses filterBy and sortBy like array.filter and array.sort.
     */
    get(filterBy?: (arrayElement: ArrayElement) => boolean, sortBy?: (a: ArrayElement, b: ArrayElement) => any): ArrayElement[];
    /**
     * Gets a copy of the first element found with the specified filter and sort (if specified).
     * It uses filterBy and sortBy like array.filter and array.sort.
     */
    getFirst(filterBy?: (arrayElement: ArrayElement) => boolean, sortBy?: (a: ArrayElement, b: ArrayElement) => any): ArrayElement | undefined;
    /**
     * Gets a copy of the last element found with the specified filter and sort (if specified).
     * It uses filterBy and sortBy like array.filter and array.sort.
     */
    getLast(filterBy?: (arrayElement: ArrayElement) => boolean, sortBy?: (a: ArrayElement, b: ArrayElement) => any): ArrayElement | undefined;
    /**
     * Gets an array with a copy of the specified property of each element.
     * It's like filtering, and then array.map (e => e[property]).
     */
    getProperty(property: string, filterBy?: (arrayElement: ArrayElement) => boolean): any[];
    /**
     * 	Count how many items are in the array with the filter specified.
     * 	it uses filterBy and sortBy like array.filter.
     */
    count(filterBy?: (arrayElement: ArrayElement) => boolean): number;
    /**
     *  Pushes one or more objects to the array.
     *  It works like array.push, only with objects.
     */
    push(contents: NewContent | NewContent[], idProp?: string): boolean;
    /**
     *  Unshifts one or more objects to the array.
     *  It works like array.unshift, only with objects.
     */
    unshift(contents: NewContent | NewContent[], idProp?: string): boolean;
    /**
     *  Insert one or more objects in the place of the index selected.
     *  It works like array.splice(index, 0, newContents).
     */
    inject(index: number, contents: NewContent | NewContent[], idProp?: string): boolean;
    /**
     *  Updates each element of the array with the specified changes.
     *  It works like object.assign for each element.
     */
    update(changes: NewContent, filterBy?: (arrayElement: ArrayElement) => boolean): boolean;
    /**
     * 	Modifies each component applying the modifier function. It can apply a filter.
     * 	For example, updateWithFunction( (e) => e.number++, (e) => e.index > 50) will
     * 	increase by one every the number property for each component which index is > 50.
     */
    updateWithFunction(fnUpdate: (arrayElement: ArrayElement) => ArrayElement, filterBy?: (arrayElement: ArrayElement) => boolean): boolean;
    /**
     *  Move the element with the specified index in the specified direction.
     *  For example, moving -1 will make it switch place with the previous object.
     *  Moving it 5 will move it from index 1 to index 6.
     */
    moveIndex(index: number, howMany: number): boolean;
    /**
     *  Switch two elements position. If you choose 2 and 5, element in the fifth position will go to the second and viceversa.
     */
    switchIndex(firstIndex: number, secondIndex: number): boolean;
    /**
     * Removes all the elements that satisfies the condition.
     * For example: remove( (e) => e.index > 50) will remove all the elements with index > 50.
     */
    remove(condition?: (arrayElement: ArrayElement) => boolean): boolean;
    /**
     * 	Replace the array content with the new contents. It's like creating a new array with the specified content.
     */
    replace(contents: NewContent | NewContent[], idProp?: string): boolean;
    /**
     * 	Sorts the array. It works like array.sort.
     */
    sort(sortBy?: (a: ArrayElement, b: ArrayElement) => any): boolean;
    /**
     * 	Empties the array.
     */
    empty(): boolean;
    /**
     *  Converts an array result to object from [{id, ...properties}, {id, ...properties}, {id, ...properties}] to {id: {id, ...properties}, id: {id, ...properties}, id:{id, ...properties}}.
     */
    convertResultToObject(arrayResult: ArrayElement[]): Record<string, any>;
    /**
     * Filter the elements and then get the IDs.
     */
    private _getIds;
    /**
     * 	Validate the new contents to insert in the array.
     */
    private _validateNewContents;
    /**
     *  Throws an error if any of the objects don't have an "id" property.
     */
    private _checkAllHaveId;
}
export default Array;
