"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * 	ARRAY:
 * 	It interacts with Redux as if it were an array of objects.
 */
var lodash_1 = require("lodash");
var reduxArrays_1 = __importDefault(require("./redux/reduxArrays"));
var Array = /** @class */ (function () {
    function Array(name) {
        this._name = name;
    }
    /**
     * Gets a copy of the original array content.
     * It uses filterBy and sortBy like array.filter and array.sort.
     */
    Array.prototype.get = function (filterBy, sortBy) {
        var state = reduxArrays_1.default.get(this._name);
        var content = lodash_1.cloneDeep(state);
        if (filterBy) {
            content = content.filter(filterBy);
        }
        if (sortBy) {
            content = content.sort(sortBy);
        }
        return content;
    };
    /**
     * Gets a copy of the first element found with the specified filter and sort (if specified).
     * It uses filterBy and sortBy like array.filter and array.sort.
     */
    Array.prototype.getFirst = function (filterBy, sortBy) {
        var content = this.get(filterBy, sortBy);
        if (content.length === 0) {
            return undefined;
        }
        return content.shift();
    };
    /**
     * Gets a copy of the last element found with the specified filter and sort (if specified).
     * It uses filterBy and sortBy like array.filter and array.sort.
     */
    Array.prototype.getLast = function (filterBy, sortBy) {
        var content = this.get(filterBy, sortBy);
        if (content.length === 0) {
            return undefined;
        }
        return content.pop();
    };
    /**
     * Gets an array with a copy of the specified property of each element.
     * It's like filtering, and then array.map (e => e[property]).
     */
    Array.prototype.getProperty = function (property, filterBy) {
        var content = this.get(filterBy);
        return content.map(function (e) {
            return e[property];
        });
    };
    /**
     * 	Count how many items are in the array with the filter specified.
     * 	it uses filterBy and sortBy like array.filter.
     */
    Array.prototype.count = function (filterBy) {
        return this.get(filterBy).length;
    };
    /**
     *  Pushes one or more objects to the array.
     *  It works like array.push, only with objects.
     */
    Array.prototype.push = function (contents, idProp) {
        if (idProp === void 0) { idProp = 'id'; }
        var validatedContents = this._validateNewContents(contents, idProp);
        return reduxArrays_1.default.push(this._name, validatedContents);
    };
    /**
     *  Unshifts one or more objects to the array.
     *  It works like array.unshift, only with objects.
     */
    Array.prototype.unshift = function (contents, idProp) {
        if (idProp === void 0) { idProp = 'id'; }
        var validatedContents = this._validateNewContents(contents, idProp);
        return reduxArrays_1.default.unshift(this._name, validatedContents);
    };
    /**
     *  Insert one or more objects in the place of the index selected.
     *  It works like array.splice(index, 0, newContents).
     */
    Array.prototype.inject = function (index, contents, idProp) {
        if (idProp === void 0) { idProp = 'id'; }
        var validatedContents = this._validateNewContents(contents, idProp);
        return reduxArrays_1.default.inject(this._name, index, validatedContents);
    };
    /**
     *  Updates each element of the array with the specified changes.
     *  It works like object.assign for each element.
     */
    Array.prototype.update = function (changes, filterBy) {
        var ids = this._getIds(filterBy);
        return reduxArrays_1.default.update(this._name, ids, changes);
    };
    /**
     * 	Modifies each component applying the modifier function. It can apply a filter.
     * 	For example, updateWithFunction( (e) => e.number++, (e) => e.index > 50) will
     * 	increase by one every the number property for each component which index is > 50.
     */
    Array.prototype.updateWithFunction = function (fnUpdate, filterBy) {
        var newContent = this.get();
        var idsToModify = this._getIds(filterBy);
        newContent.forEach(function (item, index) {
            if (!idsToModify.includes(item.id)) {
                return true;
            }
            newContent[index] = fnUpdate(item);
            return true;
        });
        this._checkAllHaveId(newContent);
        return reduxArrays_1.default.replace(this._name, newContent);
    };
    /**
     *  Move the element with the specified index in the specified direction.
     *  For example, moving -1 will make it switch place with the previous object.
     *  Moving it 5 will move it from index 1 to index 6.
     */
    Array.prototype.moveIndex = function (index, howMany) {
        var arrayState = this.get();
        var newIndex = index + howMany;
        if (typeof arrayState[index] === 'undefined' || typeof arrayState[newIndex] === 'undefined') {
            return false;
        }
        var movedElement = arrayState.splice(index, 1).shift();
        arrayState.splice(newIndex, 0, movedElement);
        return this.replace(arrayState);
    };
    /**
     *  Switch two elements position. If you choose 2 and 5, element in the fifth position will go to the second and viceversa.
     */
    Array.prototype.switchIndex = function (firstIndex, secondIndex) {
        var arrayState = this.get();
        if (typeof arrayState[firstIndex] === 'undefined' ||
            typeof arrayState[secondIndex] === 'undefined') {
            return false;
        }
        var tmpElement = arrayState[firstIndex];
        arrayState[firstIndex] = arrayState[secondIndex];
        arrayState[secondIndex] = tmpElement;
        return this.replace(arrayState);
    };
    /**
     * Removes all the elements that satisfies the condition.
     * For example: remove( (e) => e.index > 50) will remove all the elements with index > 50.
     */
    Array.prototype.remove = function (condition) {
        var ids = this._getIds(condition);
        return reduxArrays_1.default.remove(this._name, ids);
    };
    /**
     * 	Replace the array content with the new contents. It's like creating a new array with the specified content.
     */
    Array.prototype.replace = function (contents, idProp) {
        if (idProp === void 0) { idProp = 'id'; }
        var validatedContents = this._validateNewContents(contents, idProp);
        return reduxArrays_1.default.replace(this._name, validatedContents);
    };
    /**
     * 	Sorts the array. It works like array.sort.
     */
    Array.prototype.sort = function (sortBy) {
        var sameArraySorted = this.get(undefined, sortBy);
        return reduxArrays_1.default.replace(this._name, sameArraySorted);
    };
    /**
     * 	Empties the array.
     */
    Array.prototype.empty = function () {
        return reduxArrays_1.default.empty(this._name);
    };
    /**
     *  Converts an array result to object from [{id, ...properties}, {id, ...properties}, {id, ...properties}] to {id: {id, ...properties}, id: {id, ...properties}, id:{id, ...properties}}.
     */
    Array.prototype.convertResultToObject = function (arrayResult) {
        var finalResult = {};
        arrayResult.forEach(function (arrayElement) {
            if (typeof arrayElement.id === 'undefined') {
                throw new Error('The array result is not valid.');
            }
            finalResult[arrayElement.id] = arrayElement;
        });
        return finalResult;
    };
    //*
    //*
    //*	PRIVATE METHODS.
    //*
    //*
    /**
     * Filter the elements and then get the IDs.
     */
    Array.prototype._getIds = function (filterBy) {
        var contents = this.get(filterBy);
        return contents.map(function (e) { return e.id; });
    };
    /**
     * 	Validate the new contents to insert in the array.
     */
    Array.prototype._validateNewContents = function (contents, idProp) {
        if (idProp === void 0) { idProp = 'id'; }
        var newContent;
        if (!lodash_1.isArray(contents)) {
            newContent = [contents];
        }
        else {
            newContent = contents;
        }
        if (idProp === 'id') {
            this._checkAllHaveId(newContent);
            return newContent;
        }
        //  I have to clone the content in order not to change the original objects.
        var clonedContent = lodash_1.cloneDeep(newContent);
        clonedContent.forEach(function (e) {
            e.id = e[idProp];
            return true;
        });
        this._checkAllHaveId(clonedContent);
        return clonedContent;
    };
    /**
     *  Throws an error if any of the objects don't have an "id" property.
     */
    Array.prototype._checkAllHaveId = function (contents) {
        contents.forEach(function (content) {
            if (typeof content.id === 'undefined') {
                throw new Error('Elements in array must include an ID property.');
            }
        });
    };
    return Array;
}());
exports.default = Array;
