"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * 	DICTIONARY:
 * 	It interacts with Redux as if it were a Dictionary.
 */
var lodash_1 = require("lodash");
var reduxDictionaries_1 = __importDefault(require("./redux/reduxDictionaries"));
var Dictionary = /** @class */ (function () {
    function Dictionary(name) {
        this._name = name;
    }
    /**
     * Gets a copy of the dictionary content.
     */
    Dictionary.prototype.get = function () {
        var content = reduxDictionaries_1.default.get(this._name);
        return lodash_1.cloneDeep(content);
    };
    /**
     *  Gets the different keys of the dictionary.
     */
    Dictionary.prototype.getKeys = function () {
        return Object.keys(this.get());
    };
    /**
     * 	Gets how many keys the dictionary has.
     */
    Dictionary.prototype.count = function () {
        return this.getKeys().length;
    };
    /**
     * Sets the different contents of the dictionary. It works like Object.assign.
     */
    Dictionary.prototype.set = function (contents) {
        return reduxDictionaries_1.default.set(this._name, contents);
    };
    /**
     *  Sets a nested prop in the dictionary.
     *  Set {'prop1.subprop.subsubprop':value} will be like
     *  dictionary.prop1.subprop.subsubprop = value.
     *  Can also be called like ['prop1','subprop','subsubprop'].
     */
    Dictionary.prototype.deepSet = function (propertyPath, value) {
        var state = this.get();
        var arrPath = this._getDeepPropertyPath(propertyPath);
        var newState = this._deepSet(state, arrPath, value);
        return reduxDictionaries_1.default.set(this._name, newState);
    };
    /**
     *  Uses deepSet with many changes: {'prop1.subprop1.subsubprop1':1, 'prop2.subprop2':3}.
     *  Will change both properties in the state.
     */
    Dictionary.prototype.deepSetMany = function (changes) {
        var _this = this;
        var state = this.get();
        var arrPath;
        var tmpValue;
        Object.keys(changes).forEach(function (deepProperty) {
            arrPath = _this._getDeepPropertyPath(deepProperty);
            tmpValue = changes[deepProperty];
            state = _this._deepSet(state, arrPath, tmpValue);
            return true;
        }, this);
        return reduxDictionaries_1.default.set(this._name, state);
    };
    /**
     *  Merges the new content with the dictionary content (like lodash.merge).
     *  If you set to {name:{first, second},...others} the contents {name:third}, the result will have {name:{first, second, third}}.
     */
    Dictionary.prototype.merge = function (contents) {
        return reduxDictionaries_1.default.merge(this._name, contents);
    };
    /**
     * Removes (sets to undefined) the properties of the dictionary with the specified keys.
     */
    Dictionary.prototype.remove = function (keys) {
        return reduxDictionaries_1.default.remove(this._name, keys);
    };
    /**
     * Replace all the dictionary content with the new content (like erasing the content and setting the new content).
     */
    Dictionary.prototype.replace = function (content) {
        return reduxDictionaries_1.default.replace(this._name, content);
    };
    /**
     * Empties the content of the dictionary. Its value will be {}.
     */
    Dictionary.prototype.empty = function () {
        return this.replace({});
    };
    //*
    //*
    //* PRIVATE METHODS
    //*
    //*
    /**
     *  Get the property path from a string: 'prop1.prop2.prop3' will become ['prop1','prop2','prop3'].
     */
    Dictionary.prototype._getDeepPropertyPath = function (path) {
        var arrPath = path;
        if (Array.isArray(arrPath)) {
            return arrPath;
        }
        return arrPath.split('.');
    };
    /**
     *  Modifies the object creating the specified path.
     *  Assinging 'prop.subprop.subsubprop' = 3 to {}, will create: {prop:{subprop:{subsubprop:3}}}
     */
    Dictionary.prototype._deepSet = function (object, path, value) {
        if (path.length === 0) {
            return object;
        }
        var newObject = __assign({}, object);
        var nextProp = path.shift() || '';
        if (path.length === 0) {
            newObject[nextProp] = value;
            return newObject;
        }
        if (lodash_1.isUndefined(newObject[nextProp])) {
            newObject[nextProp] = {};
        }
        var nestedProperty = this._deepSet(newObject[nextProp], path, value);
        newObject[nextProp] = nestedProperty;
        return newObject;
    };
    return Dictionary;
}());
exports.default = Dictionary;
