"use strict";
/**
 *  REDUCER FACTORY:
 * 	It creates the reducers for the arrays.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
/**
 *  Add the indexes to the array.
 */
var addIndexes = function (contents) {
    return contents.map(function (content, index) {
        return __assign(__assign({}, content), { index: index });
    });
};
/**
 *  Throws an error if there is a duplicated id.
 */
var preventDuplicatedID = function (contents) {
    var ids = contents.map(function (e) { return e.id; });
    var hasDuplicates = lodash_1.uniq(ids).length !== ids.length;
    if (hasDuplicates) {
        var duplicatedID = ids.filter(function (e, index) { return ids.indexOf(e) !== index; }).pop();
        throw new Error("The id " + duplicatedID + " has already been used.");
    }
};
/**
 *  Set the contents of the array.
 */
var set = function (contents) {
    var finalContents = addIndexes(contents);
    preventDuplicatedID(finalContents);
    return finalContents;
};
/**
 * 	Push contents to the array. It works like array.push.
 */
var push = function (state, newContents) {
    var previousContents = lodash_1.cloneDeep(state);
    var clonedNewContents = lodash_1.cloneDeep(newContents);
    var contents = __spreadArrays(previousContents, clonedNewContents);
    var finalContents = addIndexes(contents);
    preventDuplicatedID(finalContents);
    return finalContents;
};
/**
 * 	Unfhists contents to the array. It works like array.unshift.
 */
var unshift = function (state, newContents) {
    var previousContents = lodash_1.cloneDeep(state);
    var clonedNewContents = lodash_1.cloneDeep(newContents);
    var contents = __spreadArrays(clonedNewContents, previousContents);
    var finalContents = addIndexes(contents);
    preventDuplicatedID(finalContents);
    return finalContents;
};
/**
 * 	Inject contents to the array in the specified index.
 *  The first item of the added array will be in the specified index.
 */
var inject = function (state, index, newContents) {
    var previousContents = lodash_1.cloneDeep(state);
    var clonedNewContents = lodash_1.cloneDeep(newContents);
    var firstSlicedPreviousContents = previousContents.slice(0, index - 1);
    var secondSlicedPreviousContents = previousContents.slice(index);
    var reorderedContents = __spreadArrays(firstSlicedPreviousContents, clonedNewContents, secondSlicedPreviousContents);
    var finalContents = addIndexes(reorderedContents);
    preventDuplicatedID(finalContents);
    return finalContents;
};
/**
 * 	Updates the contents of the array with the specified Ids.
 *  Then changes in the contents works like Object.assign.
 */
var update = function (state, ids, changes) {
    var clonedState = lodash_1.cloneDeep(state);
    var clonedChanges;
    if (typeof changes !== 'object') {
        throw new Error('Changes must be an object.');
    }
    if (changes === null) {
        throw new Error('Changes cannot be null.');
    }
    if (typeof changes.id !== 'undefined') {
        throw new Error('Cannot change "id" with "update" method.');
    }
    var newState = clonedState.map(function (content) {
        if (!ids.includes(content.id)) {
            return content;
        }
        //  You have to clone it each time, in order to prevent unexpected behaviours.
        clonedChanges = lodash_1.cloneDeep(changes);
        return __assign(__assign({}, content), clonedChanges);
    });
    return newState;
};
/**
 * 	Delete all the elements with the specified ids.
 */
var remove = function (state, ids) {
    var clonedState = lodash_1.cloneDeep(state);
    var filteredContent = clonedState.filter(function (e) { return !ids.includes(e.id); });
    var finalContents = addIndexes(filteredContent);
    return finalContents;
};
/**
 * 	Replace all the elements of the array with new elements.
 */
var replace = function (newContents) {
    var finalContents = addIndexes(newContents);
    preventDuplicatedID(finalContents);
    return finalContents;
};
/**
 * 	Creates a reducer for the specified array name.
 */
var createReducer = function (arrayName) {
    return function reducer(state, action) {
        if (state === void 0) { state = []; }
        switch (action.type) {
            case "ARRAY_SET_" + arrayName:
                return set(action.payload);
            case "ARRAY_PUSH_" + arrayName:
                return push(state, action.payload);
            case "ARRAY_UNSHIFT_" + arrayName:
                return unshift(state, action.payload);
            case "ARRAY_INJECT_" + arrayName:
                return inject(state, action.payload.index, action.payload.contents);
            case "ARRAY_UPDATE_" + arrayName:
                return update(state, action.payload.ids, action.payload.changes);
            case "ARRAY_DELETE_" + arrayName:
                return remove(state, action.payload);
            case "ARRAY_REPLACE_" + arrayName:
                return replace(action.payload);
            case "ARRAY_EMPTY_" + arrayName:
                return [];
            default:
                return state;
        }
    };
};
exports.default = createReducer;
